#include "colorselector.h"

void colorselector::Icolorselector(CView *anEnclosure, CBureaucrat *aSupervisor,
								short aSelection, short aCommandBase, GDHandle thedevice)
{
	CGridSelector::IGridSelector(anEnclosure, aSupervisor, 0, 0, 
									sizFIXEDLEFT, sizFIXEDTOP,
									aSelection, aCommandBase,
									16,16, BOXSIZE, BOXSIZE);
	prevhiliteitem = 0;			// No previous item hilighted
	ph = 0L;					// No palette exists
}

/******************************************************************************
 DrawItem {OVERRIDE}

		Draws the appropriate color in the rectangle.  Finds the color from
		the palette.
 ******************************************************************************/

void colorselector::DrawItem(short theItem, Rect *theBox)
{
	RGBColor oldcolor;
	GDHandle curdevice;
	RGBColor curcolor;
	short pixdepth;
	
	Lock(TRUE);
	if (ph)
	{
		// If theItem is withing the number of entries in the window's palette, draw
		// the rectangle in the appropriate color.  Otherwise, draw it as black.
		if ((theItem-1) < (**ph).pmEntries)
		{
			GetEntryColor(ph, theItem-1, &curcolor);
		}
		else
		{
			curcolor.red = curcolor.green = curcolor.blue = 0;
		}
		GetForeColor ( &oldcolor );
		RGBForeColor( &curcolor );
		PaintRect(theBox);
		RGBForeColor( &oldcolor );
	}
	else
	{
		// No palette associated with the window.  Draw the first entry as white
		// and the others as black.
		if (theItem == 1)
			curcolor.red = curcolor.green = curcolor.blue = 0xFFFF;	// white
		else
			curcolor.red = curcolor.green = curcolor.blue = 0;		// black
		GetForeColor ( &oldcolor );
		RGBForeColor( &curcolor );
		PaintRect(theBox);
		RGBForeColor( &oldcolor );
	}
	Lock(FALSE);
}

/******************************************************************************
 HiliteItem {OVERRIDE}

		The default hiliting method used by GridSelector is to invert the item.
		This doesn't look good with colors, so we draw a rectangle around
		the item.
 ******************************************************************************/

void	colorselector::HiliteItem(
	short		theItem,				/* Item number to hilite			*/
	HiliteState	state)					/* State of hiliting desired		*/
{
	Rect			theRect;
	register Rect	*theBox;
	long			ticks;
	
	if (theItem == 0)
		return;
	if (prevhiliteitem != 0)
	{
		// Unhilite the previous item
		FindItemBox(prevhiliteitem, &theRect);
		DrawItem(prevhiliteitem, &theRect);
		InsetRect(&theRect, -1, -1);
		FrameRect(&theRect);
	}
	FindItemBox(theItem, &theRect);		// Get the item's rectangle
	theBox = &theRect;
	
	// Draw a white rectangle inside of a black rectangle around the color
	InsetRect(theBox, -1, -1);
	ForeColor(whiteColor);
	FrameRect(theBox);
	ForeColor(blackColor);
	InsetRect(theBox, 1, 1);
	FrameRect(theBox);
	
	prevhiliteitem = theItem;
}

/******************************************************************************
 currentcolor 
 
		Returns currently selected color.

 ******************************************************************************/

void colorselector::currentcolor(RGBColor *thecolor, short *theentry)
{
	GDHandle curdevice;
	
	Lock(TRUE);
	*theentry = selection;
	if (ph)
	{
		if ((selection-1) < (**ph).pmEntries)
		{
			GetEntryColor(ph, (selection-1), thecolor);
		}
		else
		{
			thecolor->red = thecolor->green = thecolor->blue = 0;
		}
	}
	else
	{
		if (selection == 1)
			thecolor->red = thecolor->green = thecolor->blue = 0xFFFF;
		else
			thecolor->red = thecolor->green = thecolor->blue = 0;
	}
	Lock(FALSE);
}
/******************************************************************************
 windowactivated 
 
		This function is called by a document window when the window is
		activated.  It copies the palette associated with the window and
		associates it with the floating color selection window so that the
		correct colors are displayed.

 ******************************************************************************/

void colorselector::windowactivated(WindowPtr thewindow)
{
	PaletteHandle newph;
	Rect tmprect;
	GrafPtr		savePort;				/* The current port					*/
	GetPort(&savePort);					/* Save the original port			*/
	SetPort(macPort);
	Lock(TRUE);
	// Get palette associated with the activated document window
	ph = GetPalette(macPort);		// Get our window's palette if any
	if (ph)
	{
		DisposePalette(ph);			// Get rid of it
		ph = 0L;
	}
	newph = GetPalette(thewindow);
	if (newph)
	{
		ph = newph;
		HandToHand ( &ph );				// Make a copy of the palette
		SetPalette(macPort, ph, TRUE);	// associate the palette with our selection window
		ActivatePalette(macPort);
	}
	Lock(FALSE);
	InvalRect(&(macPort->portRect));	// Make sure the window gets redrawn
	SetPort(savePort);					/* Restore the original port		*/
}

void colorselector::changecolor(short thecolor)
{
	GrafPtr		savePort;				/* The current port					*/
	
	GetPort(&savePort);					/* Save the original port			*/
	Prepare();							/* Set up Window's port				*/
	ChangeSelection(thecolor);
	SetPort(savePort);					/* Restore the original port		*/
	ForceNextPrepare();
}
