#include <CObject.h>
#include "genericbox.h"
#include "colorselector.h"

extern colorselector *gcolorselector;	// The color selector pane

/******************************************************************************
 Igenericbox
 
 		Initialize a generic box.  The box's rectangle, color, and its color
 		index in the palette are saved.  We keep the palette index as well as
 		the RGB color itself because when we want to tell the color selection
 		window to display the color of this box there is no convenient way to
 		go from an RGB color to a palette index.
 ******************************************************************************/
void genericbox::Igenericbox(Rect *therect, RGBColor *thecolor, short colorentry)
{
	boxrect = *therect;
	boxcolor = *thecolor;
	this->colorentry = colorentry;
}

/******************************************************************************
 draw
 
		Draws a generic box by painting a rectangle in the box's color.
 ******************************************************************************/
void genericbox::draw(void)
{
	Lock(TRUE);
	RGBForeColor(&boxcolor);
	PaintRect(&boxrect);
	Lock(FALSE);
}

/******************************************************************************
 testhit
 
		Returns true if the point is inside the box.
 ******************************************************************************/
Boolean genericbox::testhit(Point mouseloc)
{
	Boolean result;
	
	Lock(TRUE);
	result = PtInRect ( mouseloc, &boxrect );
	Lock(FALSE);
	return result;
}

/******************************************************************************
 doselectstuff
 
		This method is invoked when the user has selected this box by clicking
		on it.  It sends a message to the color selection window to change the
		currently selected color to the color of this box.
 ******************************************************************************/
void genericbox::doselectstuff(void)
{
	Lock(TRUE);
	EraseRect(&boxrect);						// Erase the box (& redraw it later to
												// make it flash)
	gcolorselector->changecolor(colorentry);	// Change selected color to this box's
	RGBForeColor(&boxcolor);
	PaintRect(&boxrect);
	Lock(FALSE);
}