/****
 * mydoc.c
 *
 *	Document methods for a typical application.
 *
 *  Copyright  1990 Symantec Corporation.  All rights reserved.
 *
 ****/

#include "Global.h"
#include "Commands.h"
#include "CApplication.h"
#include "CBartender.h"
#include "CDataFile.h"
#include "CDecorator.h"
#include "CDesktop.h"
#include "CError.h"
#include "CPanorama.h"
#include "CScrollPane.h"
#include "mydoc.h"
#include "mypane.h"
#include "TBUtilities.h"
#include <Packages.h>
#include <CPaneBorder.h>
#include "genericbox.h"
#include "colorselector.h"
#include "docwindow.h"
#include <Palettes.h>
#include <Picker.h>


#define	WINDStarter		500		/* Resource ID for WIND template */

extern	CApplication *gApplication;	/* The application */
extern	CDecorator	*gDecorator;	/* Window dressing object	*/
extern	CDesktop	*gDesktop;		/* The enclosure for all windows */
extern	CBureaucrat	*gGopher;		/* The current boss in the chain of command */
extern	OSType		gSignature;		/* The application's signature */
extern	CError		*gError;		/* The global error handler */
extern 	colorselector *gcolorselector;	// The color selector palette (pane)
extern	CBartender	*gBartender;	/* The menu handling object */

/***
 * Imydoc
 *
 *		Initialize the document by setting its window size and creating a boxlist for it.
 ***/
 

void mydoc::Imydoc(CApplication *aSupervisor, Boolean printable)

{
	CStarterDoc::IDocument(aSupervisor, printable);
	
	// Set the window size to be the printed page size
	rightbound = pageWidth;
	bottombound = pageHeight;
	SetRect ( &docsizerect, 100, 100, rightbound+15, bottombound+15 );
	
	// Create the list to hold all the boxes.
	boxlist = new boxlist;
	boxlist->IList();
}

/***
 * BuildWindow
 * Most of this is lifted from the CStarterDoc code, except for the Pane type and the
 * code to create the palettes.
 *
 ***/

void mydoc::BuildWindow (Handle theData)

{
	CScrollPane		*theScrollPane;
	mypane	*theMainPane;
	GrafPtr theport;
	PaletteHandle bp;
	RGBColor rgbc;
	long i;
	short wcount;

	itsWindow = new docwindow;
	itsWindow->IWindow(WINDStarter, FALSE, gDesktop, this);

	wcount = gDecorator->GetWCount();
	if (wcount == 1)
	{
		// Give the second window a palette of red colors
		theport = itsWindow->GetMacPort();
		bp = NewPalette(250, 0L, pmTolerant, 0);
		rgbc.blue = 0;
		rgbc.green = 0;
		for (i=0; i<250; i++)
		{
			rgbc.red = (i*65535)/250;
			SetEntryColor(bp, i, &rgbc);
		}
		SetPalette(theport, bp, TRUE);
	}
	else if (wcount == 2)
	{
		// Give the third window a palette of green colors
		theport = itsWindow->GetMacPort();
		bp = NewPalette(200, 0L, pmTolerant, 0);
		rgbc.blue = 0;
		rgbc.red = 0;
		for (i=0; i<200; i++)
		{
			rgbc.green = (i*65535)/200;
			SetEntryColor(bp, i, &rgbc);
		}
		SetPalette(theport, bp, TRUE);
	}
	itsWindow->SetSizeRect(&docsizerect);
	
    theScrollPane = new(CScrollPane);
    
	theScrollPane->IScrollPane(itsWindow, this, 10, 10, 0, 0,
								sizELASTIC, sizELASTIC,
								TRUE, TRUE, TRUE);

	theScrollPane->FitToEnclFrame(TRUE, TRUE);

	theMainPane = new(mypane);
    theMainPane->Imypane(theScrollPane, this, rightbound, bottombound, 0, 0, 
    							sizELASTIC, sizELASTIC);
	itsMainPane = theMainPane;
	itsGopher = theMainPane;
	
	theMainPane->FitToEnclosure(TRUE, TRUE);
	theScrollPane->InstallPanorama(theMainPane);
	
	gDecorator->PlaceNewWindow(itsWindow);
	theMainPane->SetWantsClicks (true);
}

/***
 * selectabox
 *
 *		Returns true if a box was selected (clicked in) and false otherwise.
 *		If a box was selected, its doselectstuff method is called.
 ***/
Boolean mydoc::selectabox(Point testpoint)
{
	genericbox *gbox;
	
	gbox = 0;
	Lock(TRUE);
	
	// Ask the boxlist to check each box.  It will return the box object which was selected
	// or null if none was selected.
	if ((gbox = boxlist->hitbox(testpoint)) != 0L)		
	{
		gbox->doselectstuff();
	}
	Lock(FALSE);
	return (gbox != 0);
}

/***
 * DoNewBox
 *
 *		Create a new box.
 ***/
void mydoc::DoNewBox(Rect *boxrect)
{
	genericbox *gbox;
	RGBColor thecolor;
	short colorentry;
	
	Lock(TRUE);
	
	// Get the currently selected color from the color selection palette
	gcolorselector->currentcolor(&thecolor, &colorentry);
	
	gbox = new genericbox;
	gbox->Igenericbox(boxrect, &thecolor, colorentry);
	gbox->draw();
	boxlist->Append((CObject *)gbox);
	Lock(FALSE);
}

void mydoc::DoCommand(long theCommand)

{
	if (HiShort(-theCommand) == cmdColorSelect)
	{
		// Color selection change.  The low word has the current color number.
		// The currentcolor method of gColorselector could also be called here
		// to get the current entry number and RGB value.
		// This would be the place where we might check for
		// selected boxes and change their color.  Right now, we don't do anything.
		FlashMenuBar(0);		// Just to see something happen
		FlashMenuBar(0);
	}
	else
	{
		inherited::DoCommand(theCommand);
	}
}
