/****
 * mypane.c
 *
 *
 ****/
 
/**
 *
 * Initialization method.  Simply calls the superclass's initialization method.
 *
 **/

#include "mypane.h"
#include "mydoc.h"
#include "genericbox.h"

void Perform_draw(genericbox *thebox);

void mypane::Imypane(CView *anEnclosure, CBureaucrat *aSupervisor,
							short aWidth, short aHeight,
							short aHEncl, short aVEncl,
							SizingOption aHSizing, SizingOption aVSizing)
{
	CStarterPane::IStarterPane(anEnclosure, aSupervisor, aWidth, aHeight,
							aHEncl, aVEncl, aHSizing, aVSizing);
}

/***
Perform_draw
Iteration procedure for drawing each box in the box list.  It is called by
the DoForEach method of the boxlist object.  See the Draw function here for
more information.
***/
static void Perform_draw(genericbox *thebox)
{
	thebox->draw();
}

/***
 * Draw 
 *
 *	Draw the contents of the pane.
 ***/

void mypane::Draw(Rect *area)

{
	// Sends a DoForEach message to the boxlist object belonging to
	// the pane's supervisor (the document).  Object-oriented purists
	// will undoubtedly gag at referencing an instance variable belonging
	// to another class.  The "pure" way to do this would be to send a
	// message to the document requesting that it call the DoForEach
	// method of its boxlist, but that would require another method in
	// the document class.
	((mydoc *)itsSupervisor)->boxlist->DoForEach(Perform_draw);
}


/***
 * DoClick
 *
 *	The mouse went down in the pane.
 *	If the click was in an existing box, select the box.
 *	Otherwise, draw a new one.
 *
 ***/ 

void mypane::DoClick(Point hitPt, short modifierKeys, long when)

{
	/* what happens when the mouse goes down */
	Rect tmprect;
	Point dragpt,prevdragpt;
	Boolean first;
	CursHandle crosscursor;
	
	if (!(((mydoc *)itsSupervisor)->selectabox(hitPt)))
	{
		// Didn't select one.
		// Track the mouse, drawing a "rubber band" rectangle.
		first = TRUE;
		prevdragpt = hitPt;
		PenMode ( patXor );
		SetRect ( &tmprect, hitPt.h, hitPt.v, hitPt.h+100, hitPt.v+100 );
		crosscursor = GetCursor ( crossCursor );
		SetCursor ( *crosscursor );
		while (StillDown())
		{
			GetMouse(&dragpt);
			if (!EqualPt(dragpt,prevdragpt))
			{
				if (!first)
				{
					// Erase the previous rectangle
					SetRect ( &tmprect, hitPt.h, hitPt.v, prevdragpt.h, prevdragpt.v );
					FrameRect(&tmprect);
				}
				first = FALSE;
				// Draw the new rectangle
				SetRect ( &tmprect, hitPt.h, hitPt.v, dragpt.h, dragpt.v );
				FrameRect(&tmprect);
				prevdragpt = dragpt;
			}
		}
		if (!first) FrameRect(&tmprect);	// Erase the last one
		InitCursor (  );
		PenMode(patCopy);
		Lock(TRUE);
		((mydoc *)itsSupervisor)->DoNewBox(&tmprect);
		Lock(FALSE);
	}
}


/******************************************************************************
 ChangeSize

		Change the size. If necessary, shift drawing so
		that it always completely covers the frame.  (This code is copied from
		the Art Class sample application.  It makes scrolling work correctly.)
 ******************************************************************************/

void	mypane::ChangeSize(
	register Rect		*delta,			/* Movement for each side			*/
	Boolean				redraw)			/* Redraw Pane or not?				*/
{
	short		hShift;
	short		vShift;
	
										/* If new size would cause the		*/
										/*   frame to extend beyond the		*/
										/*   bounds of the diagram, we		*/
										/*   have to shift the diagram.	*/
	hShift = Max(frame.right + delta->right - bounds.right, 0);
	vShift = Max(frame.bottom + delta->bottom - bounds.bottom, 0);

	inherited::ChangeSize(delta, redraw);
	
	if ( (hShift > 0)  ||  (vShift > 0) ) {
		Scroll(-hShift, -vShift, false);
		if (redraw) {
			Refresh();
		}
	}
}
